-- The following statements assume incremental ACME database creation
-- started in Chapter 2 after creating an empty database using empty_acme.sql script.

CREATE TABLE #tmp_customer_order_totals
(
customer_name VARCHAR(30),
customer_total MONEY
)
GO

CREATE TABLE ##tmp_customer_order_totals
(
customer_name VARCHAR(30),
customer_total MONEY
)
GO

CREATE TABLE salesman
(
salesman_id_n INT CONSTRAINT pk_salesmanprim PRIMARY KEY,
salesman_code_s VARCHAR (2) CONSTRAINT uk_salescode UNIQUE,
salesman_name_s VARCHAR (50) NOT NULL,
salesman_status_s CHAR (1) CONSTRAINT chk_salesstatus CHECK
(salesman_status_s in ('N', 'Y'))
)
GO

CREATE TABLE address
(
addr_id_n INT CONSTRAINT pk_addrprimary PRIMARY KEY,
addr_custid_fn INT,
addr_salesmanid_fn INT CONSTRAINT fk_addr_salesman 
                            REFERENCES salesman (salesman_id_n),
addr_address_s VARCHAR(60),
addr_type_s VARCHAR(8) CONSTRAINT chk_addr_type CHECK
                          (addr_type_s IN ('BILLING', 'SHIPPING')),
addr_city_s VARCHAR(18) CONSTRAINT nn_addr_city NOT NULL,
addr_state_s CHAR(2),
addr_zip_s VARCHAR(10) NOT NULL,
addr_country_s CHAR(3)
)
GO



CREATE TABLE product
(
prod_id_n INTEGER NOT NULL,
prod_price_n DECIMAL(10,2),
prod_num_s VARCHAR(10),
prod_description_s VARCHAR(44) NOT NULL,
prod_status_s CHAR(1) DEFAULT 'Y',
prod_brand_s VARCHAR(20) NOT NULL,
prod_pltwid_n DECIMAL(5,2) NOT NULL,
prod_pltlen_s DECIMAL(5,2) NOT NULL,
prod_netwght_n DECIMAL(10,3),
prod_shipweight_n DECIMAL(10,3)
)
GO

INSERT INTO product
( prod_id_n,
prod_price_n,
prod_num_s,
prod_description_s,
prod_brand_s,
prod_pltwid_n,
prod_pltlen_s,
prod_netwght_n,
prod_shipweight_n
)
VALUES
(990,18.24,'990','SPRUCE LUMBER 30X40X50',
'SPRUCE LUMBER',4,6,21.22577,24.22577)
GO

SELECT prod_id_n,
prod_price_n,
prod_status_s
FROM product
GO

CREATE TABLE customer
(
cust_id_n INT NOT NULL,
cust_paytermsid_fn INT,
cust_salesmanid_fn INT,
cust_status_s VARCHAR(1) DEFAULT 'Y' NOT NULL,
cust_name_s VARCHAR(50) COLLATE FRENCH_CI_AI NOT NULL,
cust_alias_s VARCHAR(15),
cust_credhold_s VARCHAR(1) DEFAULT 'Y' NOT NULL
)
GO

CREATE TABLE order_header
(
ordhdr_id_n INTEGER NOT NULL,
ordhdr_payterms_fn INTEGER,
ordhdr_statusid_fn INTEGER,
ordhdr_custid_fn INTEGER,
ordhdr_salesmanid_fn INTEGER,
ordhdr_nbr_s VARCHAR(30) NOT NULL,
ordhdr_invoicenbr_n INTEGER,
ordhdr_orderdate_d DATETIME,
ordhdr_invoicedate_d DATETIME,
ordhdr_canceldate_d DATETIME,
ordhdr_credithold_s CHAR(1),
ordhdr_readytoinvoice_s CHAR(1) DEFAULT 'N',
ordhdr_notes_s VARCHAR(60),
ordhdr_createdby_s VARCHAR(10),
ordhdr_createdate_d DATETIME,
CONSTRAINT chk_ordhdr_ready CHECK
(ordhdr_readytoinvoice_s IN ('N', 'Y')),
CONSTRAINT chk_ordhdr_credh CHECK
(ordhdr_credithold_s IN ('N', 'Y')),
CONSTRAINT pk_ordhdrprim PRIMARY KEY (ordhdr_id_n),
CONSTRAINT idx_ordhdr_ordnbr UNIQUE (ordhdr_nbr_s)
)
GO


CREATE TABLE shipment
(
shipment_id_n INTEGER NOT NULL,
shipment_bolnum_s VARCHAR(6),
shipment_shipdate_d DATETIME,
shipment_arrivdate_d DATETIME,
shipment_totalcases_n INTEGER,
shipment_trailernbr_s VARCHAR(12),
shipment_shpmntfrght_n DECIMAL(12,2),
shipment_frtterms_s VARCHAR(3),
shipment_createdby_s VARCHAR(10),
shipment_createdate_d DATETIME,
CONSTRAINT chk_shipfrtterms CHECK
(shipment_frtterms_s IN ('COL', 'PPD')),
CONSTRAINT pk_shipmentrprim PRIMARY KEY (shipment_id_n)
)
GO


CREATE TABLE order_shipment
(
ordship_ordhdr_id_fn INTEGER NOT NULL,
ordship_shipment_id_fn INTEGER NOT NULL,
CONSTRAINT pk_ordhdrship PRIMARY KEY
(ordship_ordhdr_id_fn, ordship_shipment_id_fn),
CONSTRAINT fk_ordsh_ord FOREIGN KEY (ordship_ordhdr_id_fn)
REFERENCES order_header(ordhdr_id_n)
)
GO

INSERT INTO salesman
(
salesman_id_n,
salesman_code_s,
salesman_name_s,
salesman_status_s
)
VALUES
(23,'02','FAIRFIELD BUGS ASSOCIATION','Y')
GO


INSERT INTO address
(
addr_id_n,
addr_custid_fn,
addr_salesmanid_fn,
addr_address_s,
addr_type_s,
addr_city_s,
addr_state_s,
addr_zip_s,
addr_country_s
)
VALUES
(49,NULL,23,'223 E FLAGLER ST.',NULL,'MIAMI','FL','33131','USA')
GO


DELETE FROM salesman
GO


DROP TABLE address
GO

DROP TABLE salesman
GO

CREATE TABLE salesman
(
salesman_id_n INTEGER CONSTRAINT pk_salesmanprim PRIMARY KEY,
salesman_code_s VARCHAR(2) CONSTRAINT uk_salescode UNIQUE,
salesman_name_s VARCHAR (50) NOT NULL,
salesman_status_s CHAR (1) CONSTRAINT chk_salesstatus CHECK
(salesman_status_s in ('N', 'Y'))
)
GO

CREATE TABLE address
(
addr_id_n INTEGER CONSTRAINT pk_addrprimary PRIMARY KEY,
addr_custid_fn INTEGER,
addr_salesmanid_fn INTEGER CONSTRAINT fk_addr_salesman
REFERENCES salesman (salesman_id_n)
ON DELETE CASCADE,
addr_address_s VARCHAR(60),
addr_type_s VARCHAR(8) CONSTRAINT chk_addr_type CHECK
(addr_type_s IN ('BILLING', 'SHIPPING')),
addr_city_s VARCHAR(18) CONSTRAINT nn_addr_city NOT NULL,
addr_state_s CHAR(2),
addr_zip_s VARCHAR(10) NOT NULL,
addr_country_s CHAR(3)
)
GO

INSERT INTO salesman
(
salesman_id_n,
salesman_code_s,
salesman_name_s,
salesman_status_s
)
VALUES
(23,'02','FAIRFIELD BUGS ASSOCIATION','Y')
GO


INSERT INTO address
(
addr_id_n,
addr_custid_fn,
addr_salesmanid_fn,
addr_address_s,
addr_type_s,
addr_city_s,
addr_state_s,
addr_zip_s,
addr_country_s
)
VALUES
(49,NULL,23,'223 E FLAGLER ST.',NULL,'MIAMI','FL','33131','USA')
GO

SELECT * FROM ADDRESS
GO

DELETE FROM salesman
GO

SELECT * FROM address
GO   

CREATE TABLE phone
(
phone_id_n INTEGER NOT NULL,
phone_custid_fn INTEGER,
phone_salesmanid_fn INTEGER,
phone_phonenum_s VARCHAR(20),
phone_type_s VARCHAR(20),
CONSTRAINT chk_phone_type CHECK
(phone_type_s IN ('PHONE', 'FAX')),
CONSTRAINT pk_phonerimary PRIMARY_KEY (phone_id_n) ON INDEX01
) ON DATA01 
GO

CREATE TABLE payment_terms
(
-- The first 1 means start with; the second stands for increment by
payterms_id_n INT NOT NULL
IDENTITY (
1,1),
payterms_code_s VARCHAR(6),
payterms_desc_s VARCHAR(60),
payterms_discpct_n DECIMAL(5,2),
payterms_daystopay_n INT,
CONSTRAINT pk_payterms PRIMARY KEY (payterms_id_n)
)
GO 
 
DROP TABLE CUSTOMER
GO

CREATE TABLE customer
(
    cust_id_n             INTEGER         NOT NULL,
    cust_paytermsid_fn    INTEGER,
    cust_salesmanid_fn    INTEGER,
    cust_status_s         VARCHAR(1),
    cust_name_s           VARCHAR(50)    NOT NULL,
    cust_alias_s          VARCHAR(15),
    cust_credhold_s       VARCHAR(1)     NOT NULL  DEFAULT 'Y'
)
GO



CREATE TABLE phone
(
    phone_id_n          INTEGER           NOT NULL,
    phone_custid_fn     INTEGER,
    phone_salesmanid_fn INTEGER,
    phone_phonenum_s    VARCHAR(20),
    phone_type_s        VARCHAR(20)
)
GO



CREATE TABLE customer_phone
(
 customer_name     VARCHAR(50)    NOT NULL,
customer_phone    VARCHAR(20),
 credit_hold       VARCHAR(1)     NOT NULL  DEFAULT 'Y'
)
GO

SELECT *
INTO #PAYMENT_TERMS2
FROM PAYMENT_TERMS
GO

SELECT *
FROM phone
WHERE phone_custid_fn = 152
GO

CREATE UNIQUE INDEX idx_cust_name_als
ON customer(cust_name_s, cust_alias_s)
ON INDEX01
GO

CREATE TABLE order_line
(
    ordline_id_n            INTEGER          NOT NULL,
    ordline_ordhdrid_fn     INTEGER          NOT NULL,
    ordline_prodid_fn       INTEGER,
    ordline_ordqty_n        INTEGER,
    ordline_shipqty_n       INTEGER,
    ordline_createdate_d    DATETIME,
    ordline_createdby_s     VARCHAR(10)
)
GO

DROP TABLE PHONE
GO

CREATE TABLE phone
(
    phone_id_n            INTEGER          NOT NULL,
    phone_custid_fn       INTEGER,
    phone_salesmanid_fn   INTEGER,
    phone_phonenum_s      VARCHAR(20),
    phone_type_s          VARCHAR(20),
    CONSTRAINT chk_phone_type CHECK
              (phone_type_s IN ('PHONE', 'FAX')),
    CONSTRAINT pk_phonerimary
               PRIMARY KEY (phone_id_n)
)
GO

CREATE VIEW v_phone_number
(
  phone_id,
  phone_number
)
WITH ENCRYPTION
AS
SELECT   phone_id_n,
phone_phonenum_s
FROM     phone
WHERE    phone_type_s = 'PHONE'
WITH CHECK OPTION
GO

IF EXISTS (
 SELECT table_name
 FROM   information_schema.views
 WHERE  table_name = 'V_CUSTOMER_STATUS'
)
DROP VIEW V_CUSTOMER_STATUS
GO

CREATE VIEW v_customer_status
(
  name,
  status
)
AS
SELECT cust_name_s,
       cust_status_s
FROM   customer 
GO

CREATE VIEW v_customer_totals
(
   customer_name,
   order_number,
   total_price
)
AS
(
 SELECT   customer.cust_name_s,
          order_header.ordhdr_nbr_s,
          sum(product.prod_price_n * order_line.ordline_ordqty_n)
 FROM     customer,
          order_header,
          order_line,
          product
 WHERE    customer.cust_id_n = order_header.ordhdr_custid_fn
 AND      order_header.ordhdr_id_n = order_line.ordline_ordhdrid_fn
 AND      product.prod_id_n = order_line.ordline_prodid_fn
 AND      order_line.ordline_ordqty_n IS NOT NULL
 GROUP BY customer.cust_name_s,
          order_header.ordhdr_nbr_s
)
GO

CREATE VIEW   v_customer_totals_over_15000
AS
SELECT        *
FROM          v_customer_totals
WHERE         total_price > 15000
GO

CREATE VIEW v_contact_list
(
 name,
 phone_number,
 contact_type
)
AS
SELECT   cust_name_s,
         phone_phonenum_s,
         'customer'
FROM     customer,
         phone
WHERE    cust_id_n = phone_custid_fn
AND      phone_type_s = 'PHONE'
UNION
SELECT   salesman_name_s,
         phone_phonenum_s,
         'salesperson'
FROM     salesman,
         phone
WHERE    salesman_id_n = phone_salesmanid_fn
AND      phone_type_s = 'PHONE'
GO

CREATE VIEW v_wile_bess_orders
(
 order_number,
 order_date
)
AS
SELECT   ordhdr_nbr_s,
         ordhdr_orderdate_d
FROM     order_header
WHERE    ordhdr_custid_fn IN
   (
    SELECT  cust_id_n
    FROM    customer
    WHERE   cust_name_s = 'WILE BESS COMPANY'
   )
GO


 
DROP TABLE ADDRESS
GO

CREATE TABLE address
(
addr_id_n INT CONSTRAINT pk_addrprimary PRIMARY KEY,
addr_custid_fn INT,
addr_salesmanid_fn INT CONSTRAINT fk_addr_salesman
REFERENCES salesman (salesman_id_n)
ON DELETE CASCADE,
addr_address_s VARCHAR(60),
addr_type_s VARCHAR(8) CONSTRAINT chk_addr_type
CHECK
(addr_type_s IN ('BILLING', 'SHIPPING')),
addr_city_s VARCHAR(18) CONSTRAINT nn_addr_city NOT NULL,
addr_state_s CHAR(2),
addr_zip_s VARCHAR(10) NOT NULL,
addr_country_s CHAR(3)
)
GO

DROP TABLE salesman
GO

CREATE TABLE salesman_2
(
salesman_id_n INT CONSTRAINT pk_salesmanprim_2 PRIMARY KEY,
salesman_code_s VARCHAR (2) not null CONSTRAINT uk_salescode_2 UNIQUE,
salesman_name_s VARCHAR (50) NOT NULL,
salesman_status_s CHAR (1) CONSTRAINT chk_salesstatus_2
CHECK
(salesman_status_s in ('N', 'Y'))
)
GO

GRANT ALL ON salesman TO ACME3
GO

GRANT ALL ON ADDRESS TO ACME
GO

ALTER DATABASE acme ADD FILEGROUP DATA01
GO

ALTER DATABASE acme
ADD FILE
(
NAME = DATA0101,
FILENAME ='C:\Program Files\Microsoft SQL Server\MSSQL\Data\data0101.ndf',
SIZE = 1MB
)
TO FILEGROUP DATA01
GO 















 