-- The following statements assume incremental ACME database creation
-- started in Chapter 2 after creating an empty database using empty_acme.sql script.

CREATE TABLE virtual_column_example
(
column_a NUMBER,
column_b NUMBER,
column_c AS (column_a + column_b) VIRTUAL
);

CREATE GLOBAL TEMPORARY TABLE tmp_customer_order_totals
(
customer_name VARCHAR2(30),
customer_total NUMBER
)
ON COMMIT DELETE ROWS;

CREATE TABLE salesman
(
salesman_id_n INT CONSTRAINT pk_salesmanprim PRIMARY KEY,
salesman_code_s VARCHAR (2) CONSTRAINT uk_salescode UNIQUE,
salesman_name_s VARCHAR (50) NOT NULL,
salesman_status_s CHAR (1) CONSTRAINT chk_salesstatus CHECK
(salesman_status_s in ('N', 'Y'))
);

CREATE TABLE address
(
addr_id_n INT CONSTRAINT pk_addrprimary PRIMARY KEY,
addr_custid_fn INT,
addr_salesmanid_fn INT CONSTRAINT fk_addr_salesman
REFERENCES salesman (salesman_id_n),
addr_address_s VARCHAR(60),
addr_type_s VARCHAR(8) CONSTRAINT chk_addr_type CHECK
(addr_type_s IN ('BILLING', 'SHIPPING')),
addr_city_s VARCHAR(18) CONSTRAINT nn_addr_city NOT NULL,
addr_state_s CHAR(2),
addr_zip_s VARCHAR(10) NOT NULL,
addr_country_s CHAR(3)
);

CREATE TABLE product
(
prod_id_n INTEGER NOT NULL,
prod_price_n DECIMAL(10,2),
prod_num_s VARCHAR(10),
prod_description_s VARCHAR(44) NOT NULL,
prod_status_s CHAR(1) DEFAULT 'Y',
prod_brand_s VARCHAR(20) NOT NULL,
prod_pltwid_n DECIMAL(5,2) NOT NULL,
prod_pltlen_n DECIMAL(5,2) NOT NULL,
prod_netwght_n DECIMAL(10,3),
prod_shipweight_n DECIMAL(10,3)
);

INSERT INTO product
( prod_id_n,
prod_price_n,
prod_num_s,
prod_description_s,
prod_brand_s,
prod_pltwid_n,
prod_pltlen_n,
prod_netwght_n,
prod_shipweight_n
)
VALUES
(990,18.24,'990','SPRUCE LUMBER 30X40X50',
'SPRUCE LUMBER',4,6,21.22577,24.22577);

SELECT prod_id_n,
prod_price_n,
prod_status_s
FROM product;

CREATE TABLE order_header
(
ordhdr_id_n INTEGER NOT NULL,
ordhdr_payterms_fn INTEGER,
ordhdr_statusid_fn INTEGER,
ordhdr_custid_fn INTEGER,
ordhdr_salesmanid_fn INTEGER,
ordhdr_nbr_s VARCHAR(30) NOT NULL,
ordhdr_invoicenbr_n INTEGER,
ordhdr_orderdate_d DATE,
ordhdr_invoicedate_d DATE,
ordhdr_canceldate_d DATE,
ordhdr_credithold_s CHAR(1),
ordhdr_readytoinvoice_s CHAR(1) DEFAULT 'N',
ordhdr_notes_s VARCHAR(60),
ordhdr_createdby_s VARCHAR(10),
ordhdr_createdate_d DATE,
CONSTRAINT chk_ordhdr_ready CHECK
(ordhdr_readytoinvoice_s IN ('N', 'Y')),
CONSTRAINT chk_ordhdr_credh CHECK
(ordhdr_credithold_s IN ('N', 'Y')),
CONSTRAINT pk_ordhdrprim PRIMARY KEY (ordhdr_id_n),
CONSTRAINT idx_ordhdr_ordnbr UNIQUE (ordhdr_nbr_s)
);


CREATE TABLE shipment
(
shipment_id_n INTEGER NOT NULL,
shipment_bolnum_s VARCHAR(6),
shipment_shipdate_d DATE,
shipment_arrivdate_d DATE,
shipment_totalcases_n INTEGER,
shipment_trailernbr_s VARCHAR(12),
shipment_shpmntfrght_n DECIMAL(12,2),
shipment_frtterms_s VARCHAR(3),
shipment_createdby_s VARCHAR(10),
shipment_createdate_d DATE,
CONSTRAINT chk_shipfrtterms CHECK
(shipment_frtterms_s IN ('COL', 'PPD')),
CONSTRAINT pk_shipmentrprim PRIMARY KEY (shipment_id_n)
);


CREATE TABLE order_shipment
(
ordship_ordhdr_id_fn INTEGER NOT NULL,
ordship_shipment_id_fn INTEGER NOT NULL,
CONSTRAINT pk_ordhdrship PRIMARY KEY
(ordship_ordhdr_id_fn, ordship_shipment_id_fn),
CONSTRAINT fk_ordsh_ord FOREIGN KEY (ordship_ordhdr_id_fn)
REFERENCES order_header(ordhdr_id_n)
);

INSERT INTO salesman
(
salesman_id_n,
salesman_code_s,
salesman_name_s,
salesman_status_s
)
VALUES
(23,'02','FAIRFIELD BUGS ASSOCIATION','Y');


INSERT INTO address
(
addr_id_n,
addr_custid_fn,
addr_salesmanid_fn,
addr_address_s,
addr_type_s,
addr_city_s,
addr_state_s,
addr_zip_s,
addr_country_s
)
VALUES
(49,NULL,23,'223 E FLAGLER ST.',NULL,'MIAMI','FL','33131','USA');


DELETE FROM salesman;

DROP TABLE address;

DROP TABLE salesman;

CREATE TABLE salesman
(
salesman_id_n NUMBER CONSTRAINT pk_salesmanprim PRIMARY KEY,
salesman_code_s VARCHAR2(2) CONSTRAINT uk_salescode UNIQUE,
salesman_name_s VARCHAR2 (50) NOT NULL,
salesman_status_s CHAR (1) CONSTRAINT chk_salesstatus CHECK
(salesman_status_s in ('N', 'Y'))
);

CREATE TABLE address
(
addr_id_n NUMBER CONSTRAINT pk_addrprimary PRIMARY KEY,
addr_custid_fn NUMBER,
addr_salesmanid_fn NUMBER CONSTRAINT fk_addr_salesman
REFERENCES salesman (salesman_id_n)
ON DELETE CASCADE,
addr_address_s VARCHAR2(60),
addr_type_s VARCHAR2(8) CONSTRAINT chk_addr_type CHECK
(addr_type_s IN ('BILLING', 'SHIPPING')),
addr_city_s VARCHAR2(18) CONSTRAINT nn_addr_city NOT NULL,
addr_state_s CHAR(2),
addr_zip_s VARCHAR2(10) NOT NULL,
addr_country_s CHAR(3)
);

INSERT INTO salesman
(
salesman_id_n,
salesman_code_s,
salesman_name_s,
salesman_status_s
)
VALUES
(23,'02','FAIRFIELD BUGS ASSOCIATION','Y');


INSERT INTO address
(
addr_id_n,
addr_custid_fn,
addr_salesmanid_fn,
addr_address_s,
addr_type_s,
addr_city_s,
addr_state_s,
addr_zip_s,
addr_country_s
)
VALUES
(49,NULL,23,'223 E FLAGLER ST.',NULL,'MIAMI','FL','33131','USA');

SELECT * FROM ADDRESS;

DELETE FROM salesman;

SELECT * FROM address;

-- Create SALESMAN table with NOT DEFERRABLE CHECK constraint
-- chk salesstatus on salesman status s column
DROP TABLE address;

DROP TABLE salesman;

CREATE TABLE salesman
(
salesman_id_n NUMBER NOT NULL,
salesman_code_s VARCHAR2(2) NOT NULL,
salesman_name_s VARCHAR2(50) NOT NULL,
salesman_status_s CHAR(1) DEFAULT 'Y',
CONSTRAINT chk_salesstatus CHECK
(salesman_status_s in ('N', 'Y')),
CONSTRAINT pk_salesmanprim PRIMARY KEY (salesman_id_n)
);

INSERT INTO salesman
(
salesman_id_n,
salesman_code_s,
salesman_name_s,
salesman_status_s
)
VALUES
(23,'02','FAIRFIELD BUGS ASSOCIATION','A');

DROP TABLE salesman;

CREATE TABLE salesman
(
salesman_id_n NUMBER NOT NULL,
salesman_code_s VARCHAR2(2) NOT NULL,
salesman_name_s VARCHAR2(50) NOT NULL,
salesman_status_s CHAR(1) DEFAULT 'Y',
CONSTRAINT chk_salesstatus CHECK (salesman_status_s in ('N', 'Y'))
DEFERRABLE INITIALLY DEFERRED,
CONSTRAINT pk_salesmanprim PRIMARY KEY (salesman_id_n)
);

CREATE TABLE address
(
addr_id_n NUMBER CONSTRAINT pk_addrprimary PRIMARY KEY,
addr_custid_fn NUMBER,
addr_salesmanid_fn NUMBER CONSTRAINT fk_addr_salesman
REFERENCES salesman (salesman_id_n)
ON DELETE CASCADE,
addr_address_s VARCHAR2(60),
addr_type_s VARCHAR2(8) CONSTRAINT chk_addr_type CHECK
(addr_type_s IN ('BILLING', 'SHIPPING')),
addr_city_s VARCHAR2(18) CONSTRAINT nn_addr_city NOT NULL,
addr_state_s CHAR(2),
addr_zip_s VARCHAR2(10) NOT NULL,
addr_country_s CHAR(3)
);

INSERT INTO salesman
(
salesman_id_n,
salesman_code_s,
salesman_name_s,
salesman_status_s
)
VALUES
(23,'02','FAIRFIELD BUGS ASSOCIATION','A');

UPDATE salesman
SET salesman_status_s = 'Y'
WHERE salesman_status_s = 'A';

COMMIT;


CREATE TABLE order_line
(
ordline_id_n NUMBER NOT NULL,
ordline_ordhdrid_fn NUMBER NOT NULL,
ordline_prodid_fn NUMBER,
ordline_ordqty_n NUMBER,
ordline_shipqty_n NUMBER,
ordline_createdate_d DATE,
ordline_createdby_s VARCHAR2(10),
CONSTRAINT pk_ordlineprim PRIMARY KEY (ordline_id_n)
)
ORGANIZATION INDEX; 

CREATE TABLE payment_terms
(
    payterms_id_n          NUMBER   NOT NULL,
    payterms_code_s        VARCHAR(6),
    payterms_desc_s        VARCHAR(60),
    payterms_discpct_n     NUMBER,
    payterms_daystopay_n   NUMBER,
    CONSTRAINT pk_payterms PRIMARY KEY (payterms_id_n)
);


CREATE TABLE customer
(
    cust_id_n             NUMBER          NOT NULL,
    cust_paytermsid_fn    NUMBER,
    cust_salesmanid_fn    NUMBER,
    cust_status_s         VARCHAR2(1)     DEFAULT 'Y' NOT NULL,
    cust_name_s           VARCHAR2(50)    NOT NULL,
    cust_alias_s          VARCHAR2(15),
    cust_credhold_s       VARCHAR2(1)     DEFAULT 'Y' NOT NULL,
    CONSTRAINT chk_cust_status CHECK (cust_status_s IN ('N', 'Y')),
    CONSTRAINT chk_cust_credhold CHECK (cust_credhold_s IN ('N', 'Y')),
    CONSTRAINT pk_custprimary PRIMARY KEY (cust_id_n)
);

CREATE TABLE phone
(
    phone_id_n          NUMBER          NOT NULL,
    phone_custid_fn     NUMBER,
    phone_salesmanid_fn  NUMBER,
    phone_phonenum_s    VARCHAR2(20),
    phone_type_s        VARCHAR2(20),
    CONSTRAINT chk_phone_type CHECK (phone_type_s IN ('PHONE', 'FAX')),
    CONSTRAINT pk_phonerimary PRIMARY KEY (phone_id_n)
);
CREATE TABLE customer_phone AS
(
 SELECT      cust_name_s,
  phone_phonenum_s
 FROM        customer,
             phone
 WHERE       cust_id_n = phone_custid_fn
);

SELECT *
FROM phone
WHERE phone_custid_fn = 152;

CREATE UNIQUE INDEX idx_cust_name
ON customer(cust_name_s DESC);

CREATE INDEX idx_cust_name_status_credhold
ON customer(cust_name_s, cust_status_s, cust_credhold_s);

DROP INDEX idx_cust_name;

CREATE UNIQUE INDEX idx_cust_name
ON customer(LOWER(cust_name_s));

CREATE BITMAP INDEX idx_ordhdr_status
ON order_header (ordhdr_statusid_fn);

CREATE OR REPLACE VIEW v_customer_status
(
  name,
  status
)
AS
SELECT  cust_name_s,
        cust_status_s
FROM    customer;

CREATE VIEW v_customer_status
(
  name,
  status
)
AS
SELECT   cust_name_s,
         cust_status_s
FROM     customer;

CREATE OR REPLACE VIEW v_customer_status
AS
SELECT   cust_name_s,
         cust_status_s
FROM     customer;

CREATE OR REPLACE VIEW v_phone_number
(
  phone_id,
  phone_number
)
AS
SELECT   phone_id_n,
         phone_phonenum_s
FROM     phone
WHERE    phone_type_s = 'PHONE'
WITH READ ONLY;

UPDATE   v_phone_number
SET      phone_number = NULL
WHERE    phone_id = 1;             

CREATE VIEW v_customer_totals
(
   customer_name,
   order_number,
   total_price
)
AS
(
 SELECT   customer.cust_name_s,
          order_header.ordhdr_nbr_s,
          sum(product.prod_price_n * order_line.ordline_ordqty_n)
 FROM     customer,
          order_header,
          order_line,
          product
 WHERE    customer.cust_id_n = order_header.ordhdr_custid_fn
 AND      order_header.ordhdr_id_n = order_line.ordline_ordhdrid_fn
 AND      product.prod_id_n = order_line.ordline_prodid_fn
 AND      order_line.ordline_ordqty_n IS NOT NULL
 GROUP BY customer.cust_name_s,
          order_header.ordhdr_nbr_s
);

CREATE VIEW   v_customer_totals_over_15000
AS
SELECT        *
FROM          v_customer_totals
WHERE         total_price > 15000;

CREATE VIEW v_contact_list
(
 name,
 phone_number,
 contact_type
)
AS
SELECT   cust_name_s,
         phone_phonenum_s,
         'customer'
FROM     customer,
         phone
WHERE    cust_id_n = phone_custid_fn
AND      phone_type_s = 'PHONE'
UNION
SELECT   salesman_name_s,
         phone_phonenum_s,
         'salesperson'
FROM     salesman,
         phone
WHERE    salesman_id_n = phone_salesmanid_fn
AND      phone_type_s = 'PHONE';

CREATE VIEW v_wile_bess_orders
(
 order_number,
 order_date
)
AS
SELECT   ordhdr_nbr_s,
         ordhdr_orderdate_d
FROM     order_header
WHERE    ordhdr_custid_fn IN
   (
    SELECT  cust_id_n
    FROM    customer
    WHERE   cust_name_s = 'WILE BESS COMPANY'
   );

CREATE SEQUENCE my_sequence1;

CREATE SEQUENCE my_sequence2
START WITH 500
INCREMENT BY -10
MAXVALUE 500
MINVALUE 0;

CREATE SEQUENCE my_sequence3
START WITH 1000
INCREMENT BY -10
MAXVALUE 1000
MINVALUE 0
CYCLE;

CREATE SEQUENCE my_sequence4
START WITH 100
INCREMENT BY 1
MINVALUE 10
MAXVALUE 10000
CYCLE;

SELECT my_sequence4.NEXTVAL
FROM dual;

SELECT my_sequence4.CURRVAL
FROM dual;

SELECT my_sequence4.NEXTVAL
FROM dual;



CREATE MATERIALIZED VIEW vrm_orderline_summary
BUILD IMMEDIATE
AS
(
SELECT ordline_ordhdrid_fn,
SUM(ordline_ordqty_n) AS ord_qty_summary,
SUM(ordline_shipqty_n) AS ship_qty_summary,
COUNT (*) AS rowcount
FROM order_line
GROUP BY ordline_ordhdrid_fn
);

CREATE MATERIALIZED VIEW test2
REFRESH START WITH SYSDATE
NEXT (TRUNC(SYSDATE + 1)) + 6/24
AS SELECT cust_name_s,
phone_phonenum_s,
'CUSTOMER' CONTACT
FROM customer,
phone
WHERE cust_id_n = phone_custid_fn
AND phone_type_s = 'PHONE'
UNION
SELECT salesman_name_s,
phone_phonenum_s,
'SALESPERSON'
FROM salesman,
phone
WHERE salesman_id_n = phone_salesmanid_fn
AND phone_type_s = 'PHONE';

CREATE DATABASE LINK dbl_sales
CONNECT TO sales_manager IDENTIFIED BY sales123
USING 'SALES';

CREATE DIRECTORY acme
AS 'C:\sqlbib\oracle_acme';

GRANT READ ON DIRECTORY ACME TO SCOTT;

GRANT WRITE ON DIRECTORY ACME TO SCOTT;


 















