CREATE OR REPLACE PROCEDURE SP_PRINTCUSTPHONE
-- The procedure displays customer names and phone numbers
IS
 -- Declare local variables to fetch cursor values into
 v_custname VARCHAR(50);
 v_phone VARCHAR(20);
 -- Declare  cursor
 CURSOR cur_custphone IS
 SELECT name,
        phone_number
 FROM   v_contact_list
 WHERE  contact_type = 'customer';
BEGIN
 -- Open cursor
 OPEN cur_custphone;
 -- Fetch cursor for the first time
 FETCH cur_custphone
 INTO  v_custname,
         v_phone;
 -- Enable standard output
   DBMS_OUTPUT.ENABLE;
 -- Fetch records in loop until the last one is reached
 WHILE cur_custphone%FOUND
 LOOP
   FETCH cur_custphone
   INTO  v_custname,
         v_phone;
   -- Put line into the standard output
   DBMS_OUTPUT.PUT_LINE(v_custname || v_phone);
 END LOOP;
 -- Close cursor
 CLOSE cur_custphone;
END SP_PRINTCUSTPHONE;
/


CREATE PROCEDURE sp_productadd
/* This procedure adds new product to PRODUCT table */
(
 v_prodid              NUMBER,
 v_prodprice           NUMBER,
 v_prodnum             VARCHAR2,
 v_proddesc            VARCHAR2,
 v_prodstatus          CHAR,
 v_prodbrand           VARCHAR2,
 v_prodpltwid          NUMBER,
 v_prodpltlen          NUMBER,
 v_prodnetwgt          NUMBER,
 v_prodshipwgt         NUMBER
)

IS
  -- Local variable declaration
  v_prodcount NUMBER := 0;
  v_prodid_existing NUMBER;
BEGIN
  -- Check if product with this name already exists
  SELECT COUNT (*)
  INTO   v_prodcount
  FROM   product
  WHERE  UPPER(prod_description_s) = UPPER(v_proddesc);
  -- Product does not exist
  IF v_prodcount = 0 THEN
   -- Insert row into PRODUCT based on arguments passed
   INSERT INTO product
   VALUES
         (
           v_prodid,
           v_prodprice,
           v_prodnum,
           v_proddesc,
           v_prodstatus,
           v_prodbrand,
           v_prodpltwid,
           v_prodpltlen,
           v_prodnetwgt,
           v_prodshipwgt
         );
  -- Product with this name already exists
  ELSIF v_prodcount = 1 THEN
    -- Find the product's primary key number
    SELECT prod_id_n
    INTO   v_prodid_existing
    FROM   product
    WHERE  UPPER(prod_description_s) = UPPER(v_proddesc);
    -- Update the existing product with values
    -- passed as arguments
    UPDATE product
    SET    prod_price_n = v_prodprice,
           prod_description_s = v_proddesc,
           prod_status_s = v_prodstatus,
           prod_brand_s = v_prodbrand,
           prod_pltwid_n = v_prodpltwid,
           prod_pltlen_n = v_prodpltlen,
           prod_netwght_n = v_prodnetwgt,
           prod_shipweight_n = v_prodshipwgt
    WHERE  prod_id_n = v_prodid_existing;
  END IF;


-- Exception section -- the execution flow goes here
-- if an error occurs during the execution
EXCEPTION
  WHEN OTHERS THEN
    -- Enable standard output
    DBMS_OUTPUT.ENABLE;
    -- Put line into the standard output
    DBMS_OUTPUT.PUT_LINE(Error);
    -- Rollback all changes
    ROLLBACK;
END sp_productadd;
/

CREATE OR REPLACE FUNCTION uf_ordertax
(
  v_tax      NUMBER,
  v_ordnum   VARCHAR2
)
RETURN NUMBER
AS
    -- Declare local variables
    v_result NUMBER;
    v_ordamt NUMBER;
BEGIN
    -- This query performs variable v_ordamt assignment
    SELECT  total_price
    INTO    v_ordamt
    FROM    v_customer_totals
    WHERE   order_number = v_ordnum;
    -- Variable v_result is v_ordamt multiplied by tax
    v_result := v_ordamt * v_tax;
    -- Return result
    RETURN v_result;
END;
/

SELECT  ordhdr_nbr_s,
        ordhdr_orderdate_d,
        uf_ordertax(1.0725, ordhdr_nbr_s) AS amt_incl_tax
FROM    order_header
   JOIN
        customer
   ON   (ordhdr_custid_fn  = cust_id_n)
   JOIN
        address
   ON   (cust_id_n = addr_custid_fn)
WHERE   addr_state_s = 'CA';



CREATE OR REPLACE TRIGGER trbu_product
BEFORE  UPDATE OF prod_price_n ON product
FOR EACH ROW
BEGIN
    INSERT INTO  product_audit
    VALUES (:NEW.prod_id_n,
            :OLD.prod_price_n,
            :NEW.prod_price_n,
            USER,
            SYSDATE);
END;
/

